#! /usr/bin/env python
# This file is part of Bio Bug
# Author(s): Alexi Ramsey, Girlstart
# Copyright 2008
#
# Source code is derived from Girlstart's Project IT Girl Game Templates:
# http://itgirl.dreamhosters.com/itgirlgames/, http://www.girlstart.org/itgirl
# See the credits file for image sources.
#
# Bio Bug is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Bio Bug is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Bio Bug.  If not, see <http://www.gnu.org/licenses/>.

import pygame, random
from helpers import *
    


class Player(pygame.sprite.Sprite):
    # This is our player that will move around the screen controlled by the user
    
    def __init__(self, centerPoint, image = None):
        pygame.sprite.Sprite.__init__(self) 
        
        if image == None:
            self.image, self.rect = load_image('player-2.png',-1)
        else:
            self.image = image
            self.rect = image.get_rect()
        self.rect.center = centerPoint

        # Initialize the number of pellets eaten
        self.pellets = 0
        
        # Set the number of Pixels to move each time
        self.x_dist = 10
        self.y_dist = 10 
        
        self.question_count = 0
        
    def generateQuestion(self, background, screen):
        if self.question_count == 1:
            question = "Which vehicle is the most fuel efficient?"
            answer1, answer1rect = load_image("Truck.jpg")
            answer2, answer2rect = load_image("Smartcar2.jpg")
            answer3, answer3rect = load_image("Bug2.jpg")
            answer1rect.center = 200,400
            answer2rect.center = 200,250
            answer3rect.center = 200,100
            correct_answer = answer2rect

        if self.question_count == 2:
            question = "Which company sells bio-fuel?"
            answer1, answer1rect = load_image("Shell.jpg")
            answer2, answer2rect = load_image("Exxon.jpg")
            answer3, answer3rect = load_image("Chevron.jpg")
            answer1rect.center = 200,400
            answer2rect.center = 200,250
            answer3rect.center = 200,100
            correct_answer = answer1rect
            
        if self.question_count == 3:
            question = "Fossil fuels contribute to air pollution and Global Warming?"
            answer1, answer1rect = load_image("True.jpg")
            answer2, answer2rect = load_image("False.jpg")
            answer3, answer3rect = load_image("Empty space.jpg")
            answer1rect.center = 200,400
            answer2rect.center = 200,250
            answer3rect.center = 200,100
            correct_answer = answer1rect
            
        if self.question_count == 4:
            question = "Which organic material produces the most amount of energy?"
            answer1, answer1rect = load_image("Corn.jpg")
            answer2, answer2rect = load_image("Trash2.jpg")
            answer3, answer3rect = load_image("Sugar cane2.jpg")
            answer1rect.center = 200,405
            answer2rect.center = 200,245
            answer3rect.center = 200,75
            correct_answer = answer3rect
        
        self.screen = screen
        self.background = background
        
        activity_completed = False
        while not activity_completed:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    sys.exit()
            # Ask our question until they get it right
            if self.askQuestion(question, answer1, answer1rect, answer2, answer2rect, answer3, answer3rect, correct_answer):
                activity_completed = True

    def askQuestion(self, questionText, answer1, answer1rect, answer2, answer2rect, answer3, answer3rect, correctAnswerrect):
        # questionText is the string of the question
        # answer1, answer2 and answer3 are the possible answers for the user to choose
        # answer1rect, answer2rect and answer3rect are the Rects that describe where the images for
        # the answers need to be drawn.
        # correctAnswer is the correct answer
        
        # Set up the font for our question.  Redraw the background first to clear the screen
        self.screen.blit(self.background, (0, 0))
        font = pygame.font.Font("data/fonts/Fontin-Bold.ttf", 17)
        text = font.render(questionText, 1, (253, 245, 230))
        textrect = text.get_rect()
        textrect.centerx, textrect.centery = 250,500
        self.screen.blit(text, textrect)
        
        # Draw our answers to the screen
        self.screen.blit(answer1, answer1rect)
        self.screen.blit(answer2, answer2rect)
        self.screen.blit(answer3, answer3rect)
        
        pygame.display.flip()
        
        # Now let's loop waiting for the user's answer
        answered = False
        while not answered:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    sys.exit()
                elif event.type == pygame.MOUSEBUTTONDOWN:
                        if answer1rect.collidepoint(event.pos):
                            # If we're here, then the user has clicked on one of the answers.
                            answered = True
                            if answer1rect == correctAnswerrect:
                                # User answered correctly, let them know
                                correct = True
                            else:
                                correct = False
                        elif answer2rect.collidepoint(event.pos):
                            answered = True
                            if answer2rect == correctAnswerrect:
                                correct = True
                            else:
                                correct = False
                        elif answer3rect.collidepoint(event.pos):
                            answered = True
                            if answer3rect == correctAnswerrect:
                                correct = True
                            else:
                                correct = False				
        
        if correct:
            text = font.render("Correct!  Press any key to continue", 1, (253,245,230))
        else:
            text = font.render("Sorry, that's not correct.  Press any key to continue", 1, (253,245,230))
        
        self.screen.blit(self.background, (0, 0))
        textrect = text.get_rect()
        textrect.centerx, textrect.centery = 250,75
        self.screen.blit(text, textrect)
        
        pygame.display.flip()
        
        # Now, wait for the user to click a key to continue:
        answered = False
        while not answered:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    sys.exit()
                elif event.type == pygame.KEYDOWN:
                    answered = True				
        
        return correct
    
        # Initialize the direction of the player, picked up randomly
        self.direction = K_UP
    
    def update(self,key,block_group):
        xMove,yMove = 0,0
        # Set the fudge factor for smoothing corners, the more fudge the more lenient you are
        # A dufge factor greater than 1 allows the player to escape the maze!
        fudge_factor = 1

        if (key == K_RIGHT):
            xMove += self.x_dist
        elif (key == K_LEFT):
            xMove += -self.x_dist
        elif (key == K_UP):
            yMove += -self.y_dist
        elif (key == K_DOWN):
            yMove += self.y_dist

        self.rect.move_ip(xMove,yMove)

        # If we hit a block, don't move - reverse the movement
        if pygame.sprite.spritecollide(self, block_group, False):
            # Fudge the corners for smoother gameplay
            # Moves are done in pixels, before we reset the player's position
            # let's check to see if the player is near a corner and trying to turn
            # The position has been update with the desired turn and we have
            # the direction the player is moving in self.direction. We'll
            # use the fudge factor to see if there is a valid position nearby.
            # If there is, we'll update the player to continue in the same direction.
            # If not, we'll take back the move.
            xForce, yForce = 0,0
            xFudge, yFudge = 0,0
            for fudge in (range(1,fudge_factor+1)):
                if (self.direction == K_RIGHT):
                    xForce += self.x_dist
                    xFudge += (self.x_dist * fudge_factor)
                elif (self.direction == K_LEFT):
                    xForce += -self.x_dist
                    xFudge += (-self.x_dist * fudge_factor)
                elif (self.direction == K_UP):
                    yForce += -self.y_dist
                    yFudge += (-self.y_dist * fudge_factor)
                elif (self.direction == K_DOWN):
                    yForce += self.y_dist
                    yFudge += (self.y_dist * fudge_factor)
                self.rect.move_ip(xFudge,yFudge)
                # If the fudge in the same direction hits a block, take it back
                # If the fudge in the same direction and the move in the new direction
                # puts the player in a valid spot, take back the fudge (the move will
                # be taken back automatically after checking for fudge)
                # and move the player in the same direction with xForce and yForce
                # then break out of the fudging loop
                if (pygame.sprite.spritecollide(self, block_group, False)):
                    self.rect.move_ip(-xFudge,-yFudge)
                else:
                    self.rect.move_ip(xForce,yForce)
                    self.rect.move_ip(-xFudge,-yFudge)
                    break
            self.rect.move_ip(-xMove,-yMove)
        #If we fudge the move, we don't want to update the direction because you might
        # need another fudge before you actually change direction. If the move is
        # a valid move without fudging, update the direction of the player
        else:
            self.direction = key
            

    def dead_end(self,block_group):
        allowed_moves = 4

        # try moving right
        self.rect.move_ip(self.x_dist,0)
        if pygame.sprite.spritecollide(self, block_group, False): # blocked by a wall
            allowed_moves -= 1
        self.rect.move_ip(-self.x_dist,0)

        # try moving left
        self.rect.move_ip(-self.x_dist,0)
        if pygame.sprite.spritecollide(self, block_group, False): # blocked by a wall
            allowed_moves -= 1
        self.rect.move_ip(self.x_dist,0)

        # try moving up
        self.rect.move_ip(0,-self.y_dist)
        if pygame.sprite.spritecollide(self, block_group, False): # blocked by a wall
            allowed_moves -= 1
        self.rect.move_ip(0,self.y_dist)

        # try moving down
        self.rect.move_ip(0,self.y_dist)
        if pygame.sprite.spritecollide(self, block_group, False): # blocked by a wall
            allowed_moves -= 1
        self.rect.move_ip(0,-self.y_dist)

        if allowed_moves <= 1: # blocked by a wall in at least 3 directions
            return True
        else:
            return False

class Monster(pygame.sprite.Sprite):
    # This is our Monster that will move around the screen randomly
    
    def __init__(self, centerPoint, image = None):
        pygame.sprite.Sprite.__init__(self) 
        
        if image == None:
            self.image, self.rect = load_image('monster-2.png',-1)
        else:
            self.image = image
            self.rect = image.get_rect()
        self.rect.center = centerPoint

        # Initialize the direction and speed:
        self.direction = random.randint(1,4)
        self.speed = 5  # how many pixels to move each time
        self.moves = random.randint(50,100)
        self.moveCount = 0
            
    def update(self,block_group,block_group2=None):
        xMove,yMove = 0,0
        
        if self.direction==1:
            xMove = -self.speed
        elif self.direction==2:
            yMove = -self.speed
        elif self.direction==3:
            xMove = self.speed
        elif self.direction==4:
            yMove = self.speed
        
        self.rect.move_ip(xMove,yMove)
        self.moveCount += 1
        
        if pygame.sprite.spritecollideany(self, block_group) or pygame.sprite.spritecollideany(self, block_group2):
            # If we hit a block or finish line, don't move - reverse the movement
            self.rect.move_ip(-xMove,-yMove)
            self.direction = random.randint(1,4)
        elif self.moves == self.moveCount:
            # If we have moved enough, choose a new direction
            self.direction = random.randint(1,4)
            self.moves = random.randint(50,100)
            self.moveCount = 0
            
class Pellet(pygame.sprite.Sprite):
        
    def __init__(self, centerPoint, image = None):
        pygame.sprite.Sprite.__init__(self) 
        
        if image == None:
            self.image, self.rect = load_image('pellet-2.png',-1)
        else:
            self.image = image
            self.rect = image.get_rect()
        self.rect.center = centerPoint
     

class Block(pygame.sprite.Sprite):
        
    def __init__(self, centerPoint, image = None):
        pygame.sprite.Sprite.__init__(self) 
        
        if image == None:
            self.image, self.rect = load_image('block-2.png',-1)
        else:
            self.image = image
            self.rect = image.get_rect()
        self.rect.center = centerPoint

class Space(pygame.sprite.Sprite):
        
    def __init__(self, centerPoint, image = None):
        pygame.sprite.Sprite.__init__(self) 
        
        if image == None:
            self.image, self.rect = load_image('space.png',-1)
        else:
            self.image = image
            self.rect = image.get_rect()
        self.rect.center = centerPoint

class FinishLine(pygame.sprite.Sprite):
        
    def __init__(self, centerPoint, image = None):
        pygame.sprite.Sprite.__init__(self) 
        
        if image == None:
            self.image, self.rect = load_image('finish.png')
        else:
            self.image = image
            self.rect = image.get_rect()
        self.rect.center = centerPoint
        

